==> ccaudio2-0.7.9.tar.gz: ccaudio2-0.7.9/src/teltones.cpp <==
// Copyright (C) 2005 David Sugar, Tycho Softworks.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free software
// library without restriction.  Specifically, if other files instantiate
// templates or use macros or inline functions from this file, or you compile
// this file and link it with other files to produce an executable, this
// file does not by itself cause the resulting executable to be covered by
// the GNU General Public License.  This exception does not however
// invalidate any other reasons why the executable file might be covered by
// the GNU General Public License.
//
// This exception applies only to the code released under the name GNU
// ccAudio.  If you copy code from other releases into a copy of GNU
// ccAudio, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own for GNU ccAudio, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.
//
#include "private.h"
#include <cstdio>
#include "audio2.h"

using namespace ost;

TelTone::TelTone(tonekey_t *k, Level l, timeout_t duration) :
AudioTone(duration)
{
	tone = k;

	if(!tone)
	{
		complete = true;
		return;
	}

	framing = duration;
	def = tone->first;
	complete = false;
	remaining = silent = count = 0;
	level = l;
}

TelTone::~TelTone()
{
	AudioTone::cleanup();
}

bool TelTone::isComplete(void)
{
	return complete;
}

Audio::Linear TelTone::getFrame(bool endian)
{
	if(complete)
		return NULL;		

	if(count >= def->count && !remaining && !silent)
	{
		def = def->next;
		count = 0;
		if(!def)
		{
			complete = true;
			return NULL;
		}
	}

	if(!remaining && !silent)
	{
		if(count && !def->duration)
			return AudioTone::getFrame(endian);

		if(def->f2)
			dual(def->f1, def->f2, level, level);
		else
			single(def->f1, level);
		++count;
		remaining = def->duration / framing;
		if(def->silence)
			silent = (def->duration + def->silence) / framing - remaining;
		else
			silent = 0;	
	}

	if(!remaining && m1 && silent)
		reset();

	if(remaining)
		--remaining;
	else if(silent)
		--silent;

	return AudioTone::getFrame(endian);			
}

==> ccaudio2-0.9.10.tar.gz: ccaudio2-0.9.10/src/teltones.cpp <==
// Copyright (C) 2005 David Sugar, Tycho Softworks.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free software
// library without restriction.  Specifically, if other files instantiate
// templates or use macros or inline functions from this file, or you compile
// this file and link it with other files to produce an executable, this
// file does not by itself cause the resulting executable to be covered by
// the GNU General Public License.  This exception does not however
// invalidate any other reasons why the executable file might be covered by
// the GNU General Public License.
//
// This exception applies only to the code released under the name GNU
// ccAudio.  If you copy code from other releases into a copy of GNU
// ccAudio, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own for GNU ccAudio, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.
//
#include "private.h"
#include <cstdio>
#include "audio2.h"

using namespace ost;

TelTone::TelTone(tonekey_t *k, Level l, timeout_t duration) :
AudioTone(duration)
{
	tone = k;

	if(!tone)
	{
		complete = true;
		return;
	}

	framing = duration;
	def = tone->first;
	complete = false;
	remaining = silent = count = 0;
	level = l;
}

TelTone::~TelTone()
{
	AudioTone::cleanup();
}

bool TelTone::isComplete(void)
{
	return complete;
}

Audio::Linear TelTone::getFrame(void)
{
	if(complete)
		return NULL;		

	if(count >= def->count && !remaining && !silent)
	{
		def = def->next;
		count = 0;
		if(!def)
		{
			complete = true;
			return NULL;
		}
	}

	if(!remaining && !silent)
	{
		if(count && !def->duration)
			return AudioTone::getFrame();

		if(def->f2)
			dual(def->f1, def->f2, level, level);
		else
			single(def->f1, level);
		++count;
		remaining = def->duration / framing;
		if(def->silence)
			silent = (def->duration + def->silence) / framing - remaining;
		else
			silent = 0;	
	}

	if(!remaining && m1 && silent)
		reset();

	if(remaining)
		--remaining;
	else if(silent)
		--silent;

	return AudioTone::getFrame();			
}
